import subprocess


def nslookup(ip_address):
    try:
        # Run the nslookup command
        lookup_result = subprocess.run(['nslookup', ip_address], capture_output=True, text=True)

        # Check if the command was successful
        if lookup_result.returncode == 0:
            return lookup_result.stdout  # Return the output of the command
        else:
            return f"nslookup failed: {lookup_result.stderr}"  # Return the error if the command failed
    except Exception as e:
        return str(e)  # Handle exceptions


# Example usage
ip = '10.103.6.241'
output = nslookup(ip)
print(output)



#modules needed
import concurrent.futures
import sys
import os
import subprocess
from ipaddress import ip_network, IPv4Network, AddressValueError
import argparse
#external modules
from pythonping import ping
from tqdm import tqdm

#to check if script has elevated permissions
def is_elevated():
    try:
        if os.name == "nt":  #Windows
            import ctypes
            admin_status = ctypes.windll.shell32.IsUserAnAdmin()
            if admin_status == 0:  #not admin
                print(f"Admin status (Windows): {admin_status}")
            return admin_status != 0
        else:  #Linux/unix, macOS
            euid = os.geteuid()
            if euid != 0:  #not root
                print(f"Effective User ID(Unix-like): {euid}")
            return euid == 0
    except Exception as e:
        print(f"Error checking elevated permissions: {e}")
        return False

#function to run a ping
def pinger(ip, count, timeout):
    ip_str = str(ip)
    response = ping(ip_str, count=count, timeout=timeout)
    response_time = response.rtt_avg_ms
    if response.success and response.packet_loss == 0:
        hostname = nslookup(ip_str)
        return f"Ping... {ip_str} - {hostname} - Status: UP - Response time: {response_time} ms", True
    else:
        return f"Ping... {ip_str} - Status: DOWN - No Response", False

#function to run nslookup
def nslookup(ip_address):
    try:
        result = subprocess.run(["nslookup", ip_address], capture_output=True, text=True)
        if result.returncode == 0:
            output = result.stdout.splitlines()
            for line in output:
                if "name =" in line.lower() or "name:" in line.lower():
                    return line.split("=")[-1].strip() if "=" in line else line.split()[-1].strip()
            for line in output:
                if "name" in line.lower():
                    return line.split(":")[-1].strip()
            return ("Hostname not found")
        else:
            return f"nslookup failed: {result.stderr}"
    except Exception as e:
        return str(e)

#this function does a lot, sweeps subnet, generates output, and opens text file with results
def ping_sweeper(sweep_subnet, batch_size, timeout, count):
    total_up = 0
    all_results = []
    hosts_up = []
    total_ips = len(list(sweep_subnet))
    max_workers = min(64, (os.cpu_count() or 1) + 4)

    with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers) as executor: #threading
        with tqdm(total=total_ips, desc="Pinging subnet...", position=0, leave=True) as progress_bar:
            for batch in batch_ips(sweep_subnet, batch_size):
                futures = {executor.submit(pinger, ip, count, timeout): ip for ip in batch}
                for future in sorted(concurrent.futures.as_completed(futures), key=lambda x: futures[x]):
                    result, is_up = future.result()
                    all_results.append(result)
                    if is_up:
                        hosts_up.append(result)
                        total_up += 1
                    progress_bar.update(1)

    hosts_pinged = f"IPs Pinged: {total_ips}"
    hosts_respond = f"Number of Responses: {total_up}"

    print(hosts_pinged)
    tqdm.write(hosts_respond) #print statement
    print("\n".join(hosts_up))

    all_results.append(hosts_pinged)
    all_results.append(hosts_respond)
    #creating text file and writing each result on a new line
    with open("pingsweep_results.txt", "w") as f:
        for result in all_results:
            f.write(result + "\n")

    #checking OS to properly open file
    if os.name == "nt": #Windows
        subprocess.Popen(["notepad", "pingsweep_results.txt"])
    elif os.name =="posix":
        if sys.platform == "darwin": #macOS
            subprocess.Popen(["open", "pingsweep_results.txt"])
        else:
            subprocess.Popen(["xdg-open", "pingsweep_results.txt"]) #Linux/unix
    sys.exit()

#to batch the IP addresses
def batch_ips(sweep_subnet, batch_size):
    sweep_list = list(sweep_subnet)
    for i in range(0, len(sweep_list), batch_size):
        yield sweep_list[i:i + batch_size]

def get_user_input(prompt):
    try:
        while True:
            user_input = input(prompt)
            try: #validate input
                valid_input = ip_network(user_input)
                return valid_input
            except (ValueError, AddressValueError) as e:
                print(f"Invalid input: {e}")
                continue
    except KeyboardInterrupt:
        print("\nOperation interrupted. Exiting...")
        sys.exit(0)

def sweeper():
    parser = argparse.ArgumentParser(description="Python script for PingSweep.")
    parser.add_argument("-s", "--subnet", metavar="Subnet", type=str, help="Desired network to sweep in CIDR notation")
    parser.add_argument("-t", "--timeout", metavar="Timeout", type=float, default=0.5, help="Set a timeout in seconds. Default is 0.5 or 500ms")
    parser.add_argument("-c", "--count", metavar="Count", type=int, default=1, help="Amount of packets to send to each IP address. (will increase runtime)")
    args = parser.parse_args()

    try:
        sweep_subnet = ip_network(args.subnet) if args.subnet else get_user_input("Enter subnet in CIDR notation: ")
    except (ValueError, AddressValueError) as e:
        print(f"Invalid subnet{e}")
        return
    batch_size = 25
    try:
        ping_sweeper(sweep_subnet,timeout=args.timeout, count=args.count, batch_size=batch_size)
    except Exception as e:
        print(f"Error: {e}")
        print("Try again.")

#running the program
if is_elevated():
    sweeper()
else:
    print("Requires elevated permissions. Exiting...")



#modules needed
import concurrent.futures
import sys
import os
import subprocess
import platform
import argparse
from ipaddress import ip_network, AddressValueError
from datetime import datetime
#external modules
from tqdm import tqdm

#checking OS
def get_os_type():
    os_type = platform.system()
    if os_type == "Windows":
        return "Windows"
    else:
        return "Unix"

#function to run a ping
def pinger(ip, count, timeout):
    ip_str = str(ip)
    os_type = get_os_type()
    try:
        if os_type == "Windows":
            response = subprocess.run(
                ["ping", "-n", str(count), "-w", str(timeout * 1000), ip_str],
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                text=True
            )
        else: #for Unix
            response = subprocess.run(
                ["ping", "-c", str(count), "-W", str(timeout), ip_str],
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                text=True
            )
        #parse to get average response time
        if response.returncode == 0:
            response_time = None
            if os_type == "Windows":
                for line in response.stdout.split('\n'):
                    if "Average = " in line:
                        response_time = line.split("Average = ")[-1].strip()
                        break
            else:
                for line in response.stdout.split('\n'):
                    if "rtt min/avg/max/mdev" in line:
                        response_time = line.split('=')[1].split('/')[1].strip()
                        break
            if response_time:
                hostname = nslookup(ip_str)
                return f"Ping...{ip_str} - {hostname} - Status: UP - Response time: {response_time}", True
            else:
                return f"Ping... {ip_str} - Status: DOWN - No Response", False
        else:
            return f"Ping... {ip_str} - Status: DOWN - No Response", False
    except Exception as e:
        return f"Ping... {ip_str} - Status: DOWN - Error: {str(e)}", False

#function to run nslookup
def nslookup(ip_address):
    try:
        result = subprocess.run(["nslookup", ip_address], capture_output=True, text=True)
        if result.returncode == 0:
            output = result.stdout.splitlines()
            for line in output:
                if "name =" in line.lower() or "name:" in line.lower():
                    return line.split("=")[-1].strip() if "=" in line else line.split()[-1].strip()
            for line in output:
                if "name" in line.lower():
                    return line.split(":")[-1].strip()
            return "Hostname not found"
        else:
            return f"nslookup failed: {result.stderr}"
    except Exception as e:
        return str(e)

#this function does a lot, sweeps subnet, generates output, and opens text file with results
def ping_sweeper(sweep_subnet, batch_size, timeout, count):
    total_up = 0
    all_results = []
    hosts_up = []
    total_ips = len(list(sweep_subnet))
    max_workers = min(64, (os.cpu_count() or 1) + 4)

    with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers) as executor: #threading
        with tqdm(total=total_ips, desc="Pinging subnet...", position=0, leave=True) as progress_bar:
            for batch in batch_ips(sweep_subnet, batch_size):
                futures = {executor.submit(pinger, ip, count, timeout): ip for ip in batch}
                for future in sorted(concurrent.futures.as_completed(futures), key=lambda x: futures[x]):
                    result, is_up = future.result()
                    all_results.append(result)
                    if is_up:
                        hosts_up.append(result)
                        total_up += 1
                    progress_bar.update(1)

    hosts_pinged = f"IPs Pinged: {total_ips}"
    hosts_respond = f"Number of Responses: {total_up}"

    print(hosts_pinged)
    print(hosts_respond)
    print("\n".join(hosts_up))

    all_results.append(hosts_pinged)
    all_results.append(hosts_respond)

    #creating a new directory and text file with results
    timestamp = datetime.now().strftime("%Y-%m-%d_%H-%M")
    clean_timestamp = datetime.now().strftime("%m/%d/%Y - %H:%M")
    if not os.path.exists("sweep_results"):
        os.makedirs("sweep_results")
    text_file = f"sweep_results/sweep_results_{timestamp}.txt"
    with open(text_file, "w") as f:
        f.write(f"TIMESTAMP: {clean_timestamp}" + "\n" + "\n")
        f.write(hosts_pinged)
        f.write("\n" + hosts_respond + "\n" + "\n")
        for result in all_results:
            f.write(result + "\n")

    #checking OS to properly open file
    if os.name == "nt": #Windows
        subprocess.Popen(["notepad", f"sweep_results/sweep_results_{timestamp}.txt"])
    elif os.name =="posix":
        try:
            if sys.platform == "darwin": #macOS
                subprocess.Popen(["open", f"sweep_results/sweep_results_{timestamp}.txt"])
            else:
                subprocess.Popen(["xdg-open", f"sweep_results/sweep_results_{timestamp}.txt"]) #Linux/unix
        except Exception as e:
            print(f"Could not open file text file. {e}")
            print("Results are saved at sweep_results/")
    sys.exit()

#to batch the IP addresses
def batch_ips(sweep_subnet, batch_size):
    sweep_list = list(sweep_subnet)
    for i in range(0, len(sweep_list), batch_size):
        yield sweep_list[i:i + batch_size]

#function to get network from user
def get_user_input(prompt):
    try:
        while True:
            user_input = input(prompt)
            try: #validate input
                valid_input = ip_network(user_input)
                return valid_input
            except (ValueError, AddressValueError) as e:
                print(f"Invalid input: {e}")
                continue
    except KeyboardInterrupt:
        print("\nOperation interrupted. Exiting...")
        sys.exit(0)

#main function running the program
def main():
    parser = argparse.ArgumentParser(description="Python script for PingSweep.")
    parser.add_argument("-s", "--subnet", metavar="Subnet", type=str, help="Desired network to sweep in CIDR notation")
    parser.add_argument("-t", "--timeout", metavar="Timeout", type=float, default=0.25, help="Set a timeout in seconds. Default is 0.25 or 250ms")
    parser.add_argument("-c", "--count", metavar="Count", type=int, default=1, help="Amount of packets to send to each IP address. (will increase runtime)")
    args = parser.parse_args()

    try:
        sweep_subnet = ip_network(args.subnet) if args.subnet else get_user_input("Enter subnet in CIDR notation: ")
    except (ValueError, AddressValueError) as e:
        print(f"Invalid subnet{e}")
        return
    batch_size = 50
    try:
        ping_sweeper(sweep_subnet, timeout=args.timeout, count=args.count, batch_size=batch_size)
    except Exception as e:
        print(f"Error: {e}")

#running the program
if __name__ == "__main__":
    main()



from setuptools import setup, find_packages

setup(
    name="pingsweeper",
    version="0.1.1",
    packages=find_packages(),
    install_requires=["tqdm", "colorama"],
    entry_points={
        'console_scripts': [
            'pingsweeper=main.pingsweep:main',
        ],
    },
    author="Jacob Mackin",
    author_email="mackinjz412@gmail.com",
    description="A script for pinging subnets.",
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url="https://github.com/jzmack/pingsweep",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>3.6',
)
